@@ -13,7 +13,7 @@ from ..case import AWSCase
 
 class test_AsyncQueue(AWSCase):
 
-    def setup(self):
+    def setup_method(self):
         self.conn = Mock(name='connection')
         self.x = AsyncQueue(self.conn, '/url')
         self.callback = PromiseMock(name='callback')
